@@ -17,6 +17,7 @@ import (
 	"os"
 	"path/filepath"
 	"regexp"
+	"strings"
 	"sync"
 	"time"
 
@@ -70,6 +71,7 @@ type commandeer struct {
 	languages           langs.Languages
 
 	configured bool
+	paused     bool
 }
 
 func (c *commandeer) Set(key string, value interface{}) {
@@ -324,11 +326,12 @@ func (c *commandeer) loadConfig(mustHaveConfigFile, running bool) error {
 		}
 	}
 
-	themeVersionMismatch, minVersion := c.isThemeVsHugoVersionMismatch(sourceFs)
+	dir, themeVersionMismatch, minVersion := c.isThemeVsHugoVersionMismatch(sourceFs)
 
 	if themeVersionMismatch {
-		cfg.Logger.ERROR.Printf("Current theme does not support Hugo version %s. Minimum version required is %s\n",
-			helpers.CurrentHugoVersion.ReleaseVersion(), minVersion)
+		name := filepath.Base(dir)
+		cfg.Logger.ERROR.Printf("%s theme does not support Hugo version %s. Minimum version required is %s\n",
+			strings.ToUpper(name), helpers.CurrentHugoVersion.ReleaseVersion(), minVersion)
 	}
 
 	return nil
