@@ -96,7 +96,7 @@ func (cc *convertCmd) convertContents(mark rune) error {
 		return newUserError("Unsafe operation not allowed, use --unsafe or set a different output path")
 	}
 
-	c, err := initializeConfig(false, &cc.hugoBuilderCommon, cc, nil)
+	c, err := initializeConfig(true, false, &cc.hugoBuilderCommon, cc, nil)
 	if err != nil {
 		return err
 	}
