@@ -20,16 +20,25 @@ import (
 	jww "github.com/spf13/jwalterweatherman"
 )
 
-var envCmd = &cobra.Command{
-	Use:   "env",
-	Short: "Print Hugo version and environment info",
-	Long:  `Print Hugo version and environment info. This is useful in Hugo bug reports.`,
-	RunE: func(cmd *cobra.Command, args []string) error {
-		printHugoVersion()
-		jww.FEEDBACK.Printf("GOOS=%q\n", runtime.GOOS)
-		jww.FEEDBACK.Printf("GOARCH=%q\n", runtime.GOARCH)
-		jww.FEEDBACK.Printf("GOVERSION=%q\n", runtime.Version())
+var _ cmder = (*envCmd)(nil)
 
-		return nil
-	},
+type envCmd struct {
+	*baseCmd
+}
+
+func newEnvCmd() *envCmd {
+	return &envCmd{baseCmd: newBaseCmd(&cobra.Command{
+		Use:   "env",
+		Short: "Print Hugo version and environment info",
+		Long:  `Print Hugo version and environment info. This is useful in Hugo bug reports.`,
+		RunE: func(cmd *cobra.Command, args []string) error {
+			printHugoVersion()
+			jww.FEEDBACK.Printf("GOOS=%q\n", runtime.GOOS)
+			jww.FEEDBACK.Printf("GOARCH=%q\n", runtime.GOARCH)
+			jww.FEEDBACK.Printf("GOVERSION=%q\n", runtime.Version())
+
+			return nil
+		},
+	}),
+	}
 }
