@@ -20,6 +20,7 @@ import (
 	"path"
 	"path/filepath"
 	"runtime"
+	"sort"
 	"strings"
 	"sync"
 
@@ -71,11 +72,35 @@ func newCapturer(
 		numWorkers = n
 	}
 
+	// TODO(bep) the "index" vs "_index" check/strings should be moved in one place.
+	isBundleHeader := func(filename string) bool {
+		base := filepath.Base(filename)
+		name := helpers.Filename(base)
+		return isContentFile(base) && (name == "index" || name == "_index")
+	}
+
+	// Make sure that any bundle header files are processed before the others. This makes
+	// sure that any bundle head is processed before its resources.
+	sort.Slice(filenames, func(i, j int) bool {
+		a, b := filenames[i], filenames[j]
+		ac, bc := isBundleHeader(a), isBundleHeader(b)
+
+		if ac {
+			return true
+		}
+
+		if bc {
+			return false
+		}
+
+		return a < b
+	})
+
 	c := &capturer{
 		sem:            make(chan bool, numWorkers),
 		handler:        handler,
 		sourceSpec:     sourceSpec,
-		fs:             sourceSpec.Fs,
+		fs:             sourceSpec.SourceFs,
 		logger:         logger,
 		contentChanges: contentChanges,
 		seen:           make(map[string]bool),
@@ -135,11 +160,17 @@ func (c *capturer) capturePartial(filenames ...string) error {
 		switch tp {
 		case bundleLeaf:
 			if err := c.handleDir(resolvedFilename); err != nil {
-				return err
+				// Directory may have been deleted.
+				if !os.IsNotExist(err) {
+					return err
+				}
 			}
 		case bundleBranch:
 			if err := c.handleBranchDir(resolvedFilename); err != nil {
-				return err
+				// Directory may have been deleted.
+				if !os.IsNotExist(err) {
+					return err
+				}
 			}
 		default:
 			fi, err := c.resolveRealPath(resolvedFilename)
@@ -510,7 +541,7 @@ func (c *capturer) readDir(dirname string) (pathLangFileFis, error) {
 
 	dir, err := c.fs.Open(dirname)
 	if err != nil {
-		return nil, fmt.Errorf("readDir: %s", err)
+		return nil, err
 	}
 	defer dir.Close()
 	fis, err := dir.Readdir(-1)
