@@ -20,6 +20,10 @@ import (
 	"strings"
 	"sync"
 
+	bp "github.com/gohugoio/hugo/bufferpool"
+
+	"github.com/gohugoio/hugo/tpl"
+
 	"github.com/gohugoio/hugo/resource"
 
 	"github.com/gohugoio/hugo/media"
@@ -119,15 +123,15 @@ func (p *PageOutput) Render(layout ...string) template.HTML {
 	}
 
 	for _, layout := range l {
-		templ := p.s.Tmpl.Lookup(layout)
-		if templ == nil {
+		templ, found := p.s.Tmpl.Lookup(layout)
+		if !found {
 			// This is legacy from when we had only one output format and
 			// HTML templates only. Some have references to layouts without suffix.
 			// We default to good old HTML.
-			templ = p.s.Tmpl.Lookup(layout + ".html")
+			templ, found = p.s.Tmpl.Lookup(layout + ".html")
 		}
 		if templ != nil {
-			res, err := templ.ExecuteToString(p)
+			res, err := executeToString(templ, p)
 			if err != nil {
 				p.s.DistinctErrorLog.Printf("in .Render: Failed to execute template %q: %s", layout, err)
 				return template.HTML("")
@@ -140,7 +144,20 @@ func (p *PageOutput) Render(layout ...string) template.HTML {
 
 }
 
+func executeToString(templ tpl.Template, data interface{}) (string, error) {
+	b := bp.GetBuffer()
+	defer bp.PutBuffer(b)
+	if err := templ.Execute(b, data); err != nil {
+		return "", err
+	}
+	return b.String(), nil
+
+}
+
 func (p *Page) Render(layout ...string) template.HTML {
+	if p.mainPageOutput == nil {
+		panic(fmt.Sprintf("programming error: no mainPageOutput for %q", p.Path()))
+	}
 	return p.mainPageOutput.Render(layout...)
 }
 
@@ -201,7 +218,7 @@ func newOutputFormat(p *Page, f output.Format) *OutputFormat {
 func (p *PageOutput) AlternativeOutputFormats() (OutputFormats, error) {
 	var o OutputFormats
 	for _, of := range p.OutputFormats() {
-		if of.f.NotAlternative || of.f == p.outputFormat {
+		if of.f.NotAlternative || of.f.Name == p.outputFormat.Name {
 			continue
 		}
 		o = append(o, of)
@@ -262,7 +279,7 @@ func (p *PageOutput) renderResources() error {
 				// mode when the same resource is member of different page bundles.
 				p.deleteResource(i)
 			} else {
-				p.s.Log.ERROR.Printf("Failed to publish %q for page %q: %s", src.AbsSourceFilename(), p.pathOrTitle(), err)
+				p.s.Log.ERROR.Printf("Failed to publish Resource for page %q: %s", p.pathOrTitle(), err)
 			}
 		} else {
 			p.s.PathSpec.ProcessingStats.Incr(&p.s.PathSpec.ProcessingStats.Files)
