@@ -35,16 +35,18 @@ type TemplateHandler interface {
 	TemplateFinder
 	AddTemplate(name, tpl string) error
 	AddLateTemplate(name, tpl string) error
-	LoadTemplates(absPath, prefix string)
+	LoadTemplates(prefix string)
 	PrintErrors()
 
+	NewTextTemplate() TemplateParseFinder
+
 	MarkReady()
 	RebuildClone()
 }
 
 // TemplateFinder finds templates.
 type TemplateFinder interface {
-	Lookup(name string) *TemplateAdapter
+	Lookup(name string) (Template, bool)
 }
 
 // Template is the common interface between text/template and html/template.
@@ -53,6 +55,17 @@ type Template interface {
 	Name() string
 }
 
+// TemplateParser is used to parse ad-hoc templates, e.g. in the Resource chain.
+type TemplateParser interface {
+	Parse(name, tpl string) (Template, error)
+}
+
+// TemplateParseFinder provides both parsing and finding.
+type TemplateParseFinder interface {
+	TemplateParser
+	TemplateFinder
+}
+
 // TemplateExecutor adds some extras to Template.
 type TemplateExecutor interface {
 	Template
