@@ -55,7 +55,7 @@ var (
 	_ templateFuncsterTemplater = (*textTemplates)(nil)
 )
 
-// Protecting global map access (Amber)
+// Protecting  global map access (Amber)
 var amberMu sync.Mutex
 
 type templateErr struct {
@@ -70,25 +70,50 @@ type templateLoader interface {
 }
 
 type templateFuncsterTemplater interface {
+	templateFuncsterSetter
 	tpl.TemplateFinder
 	setFuncs(funcMap map[string]interface{})
+}
+
+type templateFuncsterSetter interface {
 	setTemplateFuncster(f *templateFuncster)
 }
 
 // templateHandler holds the templates in play.
 // It implements the templateLoader and tpl.TemplateHandler interfaces.
 type templateHandler struct {
+	mu sync.Mutex
+
 	// text holds all the pure text templates.
 	text *textTemplates
 	html *htmlTemplates
 
+	extTextTemplates []*textTemplate
+
 	amberFuncMap template.FuncMap
 
 	errors []*templateErr
 
+	// This is the filesystem to load the templates from. All the templates are
+	// stored in the root of this filesystem.
+	layoutsFs afero.Fs
+
 	*deps.Deps
 }
 
+// NewTextTemplate provides a text template parser that has all the Hugo
+// template funcs etc. built-in.
+func (t *templateHandler) NewTextTemplate() tpl.TemplateParseFinder {
+	t.mu.Lock()
+	t.mu.Unlock()
+
+	tt := &textTemplate{t: texttemplate.New("")}
+	t.extTextTemplates = append(t.extTextTemplates, tt)
+
+	return tt
+
+}
+
 func (t *templateHandler) addError(name string, err error) {
 	t.errors = append(t.errors, &templateErr{name, err})
 }
@@ -107,7 +132,7 @@ func (t *templateHandler) PrintErrors() {
 
 // Lookup tries to find a template with the given name in both template
 // collections: First HTML, then the plain text template collection.
-func (t *templateHandler) Lookup(name string) *tpl.TemplateAdapter {
+func (t *templateHandler) Lookup(name string) (tpl.Template, bool) {
 
 	if strings.HasPrefix(name, textTmplNamePrefix) {
 		// The caller has explicitly asked for a text template, so only look
@@ -119,8 +144,8 @@ func (t *templateHandler) Lookup(name string) *tpl.TemplateAdapter {
 	}
 
 	// Look in both
-	if te := t.html.Lookup(name); te != nil {
-		return te
+	if te, found := t.html.Lookup(name); found {
+		return te, true
 	}
 
 	return t.text.Lookup(name)
@@ -129,10 +154,11 @@ func (t *templateHandler) Lookup(name string) *tpl.TemplateAdapter {
 
 func (t *templateHandler) clone(d *deps.Deps) *templateHandler {
 	c := &templateHandler{
-		Deps:   d,
-		html:   &htmlTemplates{t: template.Must(t.html.t.Clone()), overlays: make(map[string]*template.Template)},
-		text:   &textTemplates{t: texttemplate.Must(t.text.t.Clone()), overlays: make(map[string]*texttemplate.Template)},
-		errors: make([]*templateErr, 0),
+		Deps:      d,
+		layoutsFs: d.BaseFs.Layouts.Fs,
+		html:      &htmlTemplates{t: template.Must(t.html.t.Clone()), overlays: make(map[string]*template.Template)},
+		text:      &textTemplates{textTemplate: &textTemplate{t: texttemplate.Must(t.text.t.Clone())}, overlays: make(map[string]*texttemplate.Template)},
+		errors:    make([]*templateErr, 0),
 	}
 
 	d.Tmpl = c
@@ -166,14 +192,15 @@ func newTemplateAdapter(deps *deps.Deps) *templateHandler {
 		overlays: make(map[string]*template.Template),
 	}
 	textT := &textTemplates{
-		t:        texttemplate.New(""),
-		overlays: make(map[string]*texttemplate.Template),
+		textTemplate: &textTemplate{t: texttemplate.New("")},
+		overlays:     make(map[string]*texttemplate.Template),
 	}
 	return &templateHandler{
-		Deps:   deps,
-		html:   htmlT,
-		text:   textT,
-		errors: make([]*templateErr, 0),
+		Deps:      deps,
+		layoutsFs: deps.BaseFs.Layouts.Fs,
+		html:      htmlT,
+		text:      textT,
+		errors:    make([]*templateErr, 0),
 	}
 
 }
@@ -199,24 +226,27 @@ func (t *htmlTemplates) setTemplateFuncster(f *templateFuncster) {
 	t.funcster = f
 }
 
-func (t *htmlTemplates) Lookup(name string) *tpl.TemplateAdapter {
+func (t *htmlTemplates) Lookup(name string) (tpl.Template, bool) {
 	templ := t.lookup(name)
 	if templ == nil {
-		return nil
+		return nil, false
 	}
-	return &tpl.TemplateAdapter{Template: templ, Metrics: t.funcster.Deps.Metrics}
+	return &tpl.TemplateAdapter{Template: templ, Metrics: t.funcster.Deps.Metrics}, true
 }
 
 func (t *htmlTemplates) lookup(name string) *template.Template {
-	if templ := t.t.Lookup(name); templ != nil {
-		return templ
-	}
+
+	// Need to check in the overlay registry first as it will also be found below.
 	if t.overlays != nil {
 		if templ, ok := t.overlays[name]; ok {
 			return templ
 		}
 	}
 
+	if templ := t.t.Lookup(name); templ != nil {
+		return templ
+	}
+
 	if t.clone != nil {
 		return t.clone.Lookup(name)
 	}
@@ -224,39 +254,40 @@ func (t *htmlTemplates) lookup(name string) *template.Template {
 	return nil
 }
 
+func (t *textTemplates) setTemplateFuncster(f *templateFuncster) {
+	t.funcster = f
+}
+
 type textTemplates struct {
-	funcster *templateFuncster
-
-	t *texttemplate.Template
-
+	*textTemplate
+	funcster   *templateFuncster
 	clone      *texttemplate.Template
 	cloneClone *texttemplate.Template
 
 	overlays map[string]*texttemplate.Template
 }
 
-func (t *textTemplates) setTemplateFuncster(f *templateFuncster) {
-	t.funcster = f
-}
-
-func (t *textTemplates) Lookup(name string) *tpl.TemplateAdapter {
+func (t *textTemplates) Lookup(name string) (tpl.Template, bool) {
 	templ := t.lookup(name)
 	if templ == nil {
-		return nil
+		return nil, false
 	}
-	return &tpl.TemplateAdapter{Template: templ, Metrics: t.funcster.Deps.Metrics}
+	return &tpl.TemplateAdapter{Template: templ, Metrics: t.funcster.Deps.Metrics}, true
 }
 
 func (t *textTemplates) lookup(name string) *texttemplate.Template {
-	if templ := t.t.Lookup(name); templ != nil {
-		return templ
-	}
+
+	// Need to check in the overlay registry first as it will also be found below.
 	if t.overlays != nil {
 		if templ, ok := t.overlays[name]; ok {
 			return templ
 		}
 	}
 
+	if templ := t.t.Lookup(name); templ != nil {
+		return templ
+	}
+
 	if t.clone != nil {
 		return t.clone.Lookup(name)
 	}
@@ -287,11 +318,11 @@ func (t *textTemplates) setFuncs(funcMap map[string]interface{}) {
 	t.t.Funcs(funcMap)
 }
 
-// LoadTemplates loads the templates, starting from the given absolute path.
+// LoadTemplates loads the templates from the layouts filesystem.
 // A prefix can be given to indicate a template namespace to load the templates
 // into, i.e. "_internal" etc.
-func (t *templateHandler) LoadTemplates(absPath, prefix string) {
-	t.loadTemplates(absPath, prefix)
+func (t *templateHandler) LoadTemplates(prefix string) {
+	t.loadTemplates(prefix)
 
 }
 
@@ -324,9 +355,34 @@ func (t *htmlTemplates) addLateTemplate(name, tpl string) error {
 	return t.addTemplateIn(t.clone, name, tpl)
 }
 
+type textTemplate struct {
+	t *texttemplate.Template
+}
+
+func (t *textTemplate) Parse(name, tpl string) (tpl.Template, error) {
+	return t.parSeIn(t.t, name, tpl)
+}
+
+func (t *textTemplate) Lookup(name string) (tpl.Template, bool) {
+	tpl := t.t.Lookup(name)
+	return tpl, tpl != nil
+}
+
+func (t *textTemplate) parSeIn(tt *texttemplate.Template, name, tpl string) (*texttemplate.Template, error) {
+	templ, err := tt.New(name).Parse(tpl)
+	if err != nil {
+		return nil, err
+	}
+
+	if err := applyTemplateTransformersToTextTemplate(templ); err != nil {
+		return nil, err
+	}
+	return templ, nil
+}
+
 func (t *textTemplates) addTemplateIn(tt *texttemplate.Template, name, tpl string) error {
 	name = strings.TrimPrefix(name, textTmplNamePrefix)
-	templ, err := tt.New(name).Parse(tpl)
+	templ, err := t.parSeIn(tt, name, tpl)
 	if err != nil {
 		return err
 	}
@@ -406,102 +462,71 @@ func (t *templateHandler) RebuildClone() {
 	t.text.clone = texttemplate.Must(t.text.cloneClone.Clone())
 }
 
-func (t *templateHandler) loadTemplates(absPath string, prefix string) {
-	t.Log.DEBUG.Printf("Load templates from path %q prefix %q", absPath, prefix)
+func (t *templateHandler) loadTemplates(prefix string) {
 	walker := func(path string, fi os.FileInfo, err error) error {
+		if err != nil || fi.IsDir() {
+			return nil
+		}
+
+		if isDotFile(path) || isBackupFile(path) || isBaseTemplate(path) {
+			return nil
+		}
+
+		workingDir := t.PathSpec.WorkingDir
+
+		descriptor := output.TemplateLookupDescriptor{
+			WorkingDir:    workingDir,
+			RelPath:       path,
+			Prefix:        prefix,
+			OutputFormats: t.OutputFormatsConfig,
+			FileExists: func(filename string) (bool, error) {
+				return helpers.Exists(filename, t.Layouts.Fs)
+			},
+			ContainsAny: func(filename string, subslices [][]byte) (bool, error) {
+				return helpers.FileContainsAny(filename, subslices, t.Layouts.Fs)
+			},
+		}
+
+		tplID, err := output.CreateTemplateNames(descriptor)
 		if err != nil {
+			t.Log.ERROR.Printf("Failed to resolve template in path %q: %s", path, err)
+
 			return nil
 		}
 
-		t.Log.DEBUG.Println("Template path", path)
-		if fi.Mode()&os.ModeSymlink == os.ModeSymlink {
-			link, err := filepath.EvalSymlinks(absPath)
-			if err != nil {
-				t.Log.ERROR.Printf("Cannot read symbolic link '%s', error was: %s", absPath, err)
-				return nil
-			}
-
-			linkfi, err := t.Fs.Source.Stat(link)
-			if err != nil {
-				t.Log.ERROR.Printf("Cannot stat '%s', error was: %s", link, err)
-				return nil
-			}
-
-			if !linkfi.Mode().IsRegular() {
-				t.Log.ERROR.Printf("Symbolic links for directories not supported, skipping '%s'", absPath)
-			}
-			return nil
+		if err := t.addTemplateFile(tplID.Name, tplID.MasterFilename, tplID.OverlayFilename); err != nil {
+			t.Log.ERROR.Printf("Failed to add template %q in path %q: %s", tplID.Name, path, err)
 		}
 
-		if !fi.IsDir() {
-			if isDotFile(path) || isBackupFile(path) || isBaseTemplate(path) {
-				return nil
-			}
-
-			var (
-				workingDir = t.PathSpec.WorkingDir()
-				themeDir   = t.PathSpec.GetThemeDir()
-				layoutDir  = t.PathSpec.LayoutDir()
-			)
-
-			if themeDir != "" && strings.HasPrefix(absPath, themeDir) {
-				layoutDir = "layouts"
-			}
-
-			li := strings.LastIndex(path, layoutDir) + len(layoutDir) + 1
-			relPath := path[li:]
-			templateDir := path[:li-len(layoutDir)-1]
-
-			descriptor := output.TemplateLookupDescriptor{
-				TemplateDir:   templateDir,
-				WorkingDir:    workingDir,
-				LayoutDir:     layoutDir,
-				RelPath:       relPath,
-				Prefix:        prefix,
-				ThemeDir:      themeDir,
-				OutputFormats: t.OutputFormatsConfig,
-				FileExists: func(filename string) (bool, error) {
-					return helpers.Exists(filename, t.Fs.Source)
-				},
-				ContainsAny: func(filename string, subslices [][]byte) (bool, error) {
-					return helpers.FileContainsAny(filename, subslices, t.Fs.Source)
-				},
-			}
-
-			tplID, err := output.CreateTemplateNames(descriptor)
-			if err != nil {
-				t.Log.ERROR.Printf("Failed to resolve template in path %q: %s", path, err)
-
-				return nil
-			}
-
-			if err := t.addTemplateFile(tplID.Name, tplID.MasterFilename, tplID.OverlayFilename); err != nil {
-				t.Log.ERROR.Printf("Failed to add template %q in path %q: %s", tplID.Name, path, err)
-			}
-
-		}
 		return nil
 	}
-	if err := helpers.SymbolicWalk(t.Fs.Source, absPath, walker); err != nil {
+
+	if err := helpers.SymbolicWalk(t.Layouts.Fs, "", walker); err != nil {
 		t.Log.ERROR.Printf("Failed to load templates: %s", err)
 	}
+
 }
 
 func (t *templateHandler) initFuncs() {
 
 	// Both template types will get their own funcster instance, which
 	// in the current case contains the same set of funcs.
-	for _, funcsterHolder := range []templateFuncsterTemplater{t.html, t.text} {
+	funcMap := createFuncMap(t.Deps)
+	for _, funcsterHolder := range []templateFuncsterSetter{t.html, t.text} {
 		funcster := newTemplateFuncster(t.Deps)
 
 		// The URL funcs in the funcMap is somewhat language dependent,
 		// so we need to wait until the language and site config is loaded.
-		funcster.initFuncMap()
+		funcster.initFuncMap(funcMap)
 
 		funcsterHolder.setTemplateFuncster(funcster)
 
 	}
 
+	for _, extText := range t.extTextTemplates {
+		extText.t.Funcs(funcMap)
+	}
+
 	// Amber is HTML only.
 	t.amberFuncMap = template.FuncMap{}
 
@@ -534,6 +559,7 @@ func (t *templateHandler) handleMaster(name, overlayFilename, masterFilename str
 }
 
 func (t *htmlTemplates) handleMaster(name, overlayFilename, masterFilename string, onMissing func(filename string) (string, error)) error {
+
 	masterTpl := t.lookup(masterFilename)
 
 	if masterTpl == nil {
@@ -565,6 +591,7 @@ func (t *htmlTemplates) handleMaster(name, overlayFilename, masterFilename strin
 	if err := applyTemplateTransformersToHMLTTemplate(overlayTpl); err != nil {
 		return err
 	}
+
 	t.overlays[name] = overlayTpl
 
 	return err
@@ -572,6 +599,7 @@ func (t *htmlTemplates) handleMaster(name, overlayFilename, masterFilename strin
 }
 
 func (t *textTemplates) handleMaster(name, overlayFilename, masterFilename string, onMissing func(filename string) (string, error)) error {
+
 	name = strings.TrimPrefix(name, textTmplNamePrefix)
 	masterTpl := t.lookup(masterFilename)
 
@@ -610,12 +638,16 @@ func (t *textTemplates) handleMaster(name, overlayFilename, masterFilename strin
 func (t *templateHandler) addTemplateFile(name, baseTemplatePath, path string) error {
 	t.checkState()
 
+	t.Log.DEBUG.Printf("Add template file: name %q, baseTemplatePath %q, path %q", name, baseTemplatePath, path)
+
 	getTemplate := func(filename string) (string, error) {
-		b, err := afero.ReadFile(t.Fs.Source, filename)
+		b, err := afero.ReadFile(t.Layouts.Fs, filename)
 		if err != nil {
 			return "", err
 		}
-		return string(b), nil
+		s := string(b)
+
+		return s, nil
 	}
 
 	// get the suffix and switch on that
@@ -625,7 +657,7 @@ func (t *templateHandler) addTemplateFile(name, baseTemplatePath, path string) e
 		//	Only HTML support for Amber
 		withoutExt := strings.TrimSuffix(name, filepath.Ext(name))
 		templateName := withoutExt + ".html"
-		b, err := afero.ReadFile(t.Fs.Source, path)
+		b, err := afero.ReadFile(t.Layouts.Fs, path)
 
 		if err != nil {
 			return err
@@ -654,14 +686,14 @@ func (t *templateHandler) addTemplateFile(name, baseTemplatePath, path string) e
 	case ".ace":
 		//	Only HTML support for Ace
 		var innerContent, baseContent []byte
-		innerContent, err := afero.ReadFile(t.Fs.Source, path)
+		innerContent, err := afero.ReadFile(t.Layouts.Fs, path)
 
 		if err != nil {
 			return err
 		}
 
 		if baseTemplatePath != "" {
-			baseContent, err = afero.ReadFile(t.Fs.Source, baseTemplatePath)
+			baseContent, err = afero.ReadFile(t.Layouts.Fs, baseTemplatePath)
 			if err != nil {
 				return err
 			}
@@ -680,8 +712,6 @@ func (t *templateHandler) addTemplateFile(name, baseTemplatePath, path string) e
 			return err
 		}
 
-		t.Log.DEBUG.Printf("Add template file from path %s", path)
-
 		return t.AddTemplate(name, templ)
 	}
 }
@@ -726,5 +756,5 @@ func isBackupFile(path string) bool {
 const baseFileBase = "baseof"
 
 func isBaseTemplate(path string) bool {
-	return strings.Contains(path, baseFileBase)
+	return strings.Contains(filepath.Base(path), baseFileBase)
 }
